import logging
from dataclasses import dataclass

from quorum_mininode_py.api import LightNodeAPI
from quorum_mininode_py.client._http import HttpRequest
from quorum_mininode_py.crypto import account as eth_acc
from quorum_mininode_py.crypto import age as age_acc
from quorum_mininode_py.utils.url import decode_seed_url

logger = logging.getLogger(__name__)


@dataclass
class RumGroup:
    seed: dict = None
    seed_url: str = None
    group_id: str = None
    aes_key: str = None
    encryption_type: str = None
    chain_urls: list = None

    def __init__(self, seed_url: str):
        try:
            info = decode_seed_url(seed_url)
        except KeyError as err:
            raise ValueError(f"invalid seed_url: {err}") from err
        seed = info["seed"]
        self.seed = seed
        self.seed_url = seed_url
        self.group_id = seed["group_id"]
        self.aes_key = bytes.fromhex(seed["cipher_key"])
        self.encryption_type = seed["encryption_type"]
        self.chain_urls = info["chain_urls"]


@dataclass
class RumAccount:
    pvtkey: str = None
    pubkey: str = None
    address: str = None
    age_pvtkey: str = None
    age_pubkey: str = None

    def __init__(self, pvtkey: str = None, age_pvtkey: str = None):
        self.pvtkey = pvtkey or eth_acc.create_pvtkey()
        self.pubkey = eth_acc.pvtkey_to_pubkey(self.pvtkey)
        self.address = eth_acc.pubkey_to_address(self.pubkey)

        if age_pvtkey:
            self.age_pvtkey = age_pvtkey
            self.age_pubkey = age_acc.age_pvtkey_to_pubkey(age_pvtkey)
        else:
            self.age_pvtkey, self.age_pubkey = age_acc.create_age_keypair()


class MiniNode:
    """python for quorum lightnode, without datastore, one MiniNode client for one group"""

    def __init__(self, seed_url: str, pvtkey=None, age_pvtkey=None):
        self.group = RumGroup(seed_url)
        self.http = HttpRequest(chain_urls=self.group.chain_urls or [])
        self.account = RumAccount(pvtkey, age_pvtkey)
        self.api = LightNodeAPI(self)

    def change_account(self, pvtkey, age_pvtkey=None):
        age_pvtkey = age_pvtkey or self.account.age_pvtkey
        if pvtkey != self.account.pvtkey or age_pvtkey != self.account.age_pvtkey:
            self.account = RumAccount(pvtkey, age_pvtkey)
            self.api = LightNodeAPI(self)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: quorum.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0cquorum.proto\x12\tquorum.pb"=\n\x07Package\x12$\n\x04type\x18\x01 \x01(\x0e\x32\x16.quorum.pb.PackageType\x12\x0c\n\x04\x44\x61ta\x18\x02 \x01(\x0c"\x88\x02\n\x03Trx\x12\r\n\x05TrxId\x18\x01 \x01(\t\x12 \n\x04Type\x18\x02 \x01(\x0e\x32\x12.quorum.pb.TrxType\x12\x0f\n\x07GroupId\x18\x03 \x01(\t\x12\x0c\n\x04\x44\x61ta\x18\x04 \x01(\x0c\x12\x11\n\tTimeStamp\x18\x05 \x01(\x03\x12\x0f\n\x07Version\x18\x06 \x01(\t\x12\x0f\n\x07\x45xpired\x18\x07 \x01(\x03\x12\x13\n\x0bResendCount\x18\x08 \x01(\x03\x12\r\n\x05Nonce\x18\t \x01(\x03\x12\x14\n\x0cSenderPubkey\x18\n \x01(\t\x12\x12\n\nSenderSign\x18\x0b \x01(\x0c\x12.\n\x0bStorageType\x18\x0c \x01(\x0e\x32\x19.quorum.pb.TrxStroageType"\xca\x01\n\x05\x42lock\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x0f\n\x07\x42lockId\x18\x02 \x01(\x04\x12\r\n\x05\x45poch\x18\x03 \x01(\x04\x12\x10\n\x08PrevHash\x18\x04 \x01(\x0c\x12\x16\n\x0eProducerPubkey\x18\x05 \x01(\t\x12\x1c\n\x04Trxs\x18\x06 \x03(\x0b\x32\x0e.quorum.pb.Trx\x12\x0c\n\x04Sudo\x18\x07 \x01(\x08\x12\x11\n\tTimeStamp\x18\x08 \x01(\x03\x12\x11\n\tBlockHash\x18\t \x01(\x0c\x12\x14\n\x0cProducerSign\x18\n \x01(\x0c"X\n\x08ReqBlock\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x11\n\tFromBlock\x18\x02 \x01(\x04\x12\x15\n\rBlksRequested\x18\x03 \x01(\x05\x12\x11\n\tReqPubkey\x18\x04 \x01(\t"0\n\x0c\x42locksBundle\x12 \n\x06\x42locks\x18\x01 \x03(\x0b\x32\x10.quorum.pb.Block"\xe2\x01\n\x0cReqBlockResp\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x17\n\x0fRequesterPubkey\x18\x02 \x01(\t\x12\x16\n\x0eProviderPubkey\x18\x03 \x01(\t\x12\'\n\x06Result\x18\x04 \x01(\x0e\x32\x17.quorum.pb.ReqBlkResult\x12\x11\n\tFromBlock\x18\x05 \x01(\x04\x12\x15\n\rBlksRequested\x18\x06 \x01(\x05\x12\x14\n\x0c\x42lksProvided\x18\x07 \x01(\x05\x12\'\n\x06\x42locks\x18\x08 \x01(\x0b\x32\x17.quorum.pb.BlocksBundle"S\n\x08PostItem\x12\r\n\x05TrxId\x18\x01 \x01(\t\x12\x14\n\x0cSenderPubkey\x18\x02 \x01(\t\x12\x0f\n\x07\x43ontent\x18\x03 \x01(\x0c\x12\x11\n\tTimeStamp\x18\x04 \x01(\x03"\xc9\x01\n\x0cProducerItem\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x16\n\x0eProducerPubkey\x18\x02 \x01(\t\x12\x18\n\x10GroupOwnerPubkey\x18\x03 \x01(\t\x12\x16\n\x0eGroupOwnerSign\x18\x04 \x01(\t\x12%\n\x06\x41\x63tion\x18\x05 \x01(\x0e\x32\x15.quorum.pb.ActionType\x12\x16\n\x0eWithnessBlocks\x18\x06 \x01(\x03\x12\x11\n\tTimeStamp\x18\x07 \x01(\x03\x12\x0c\n\x04Memo\x18\x08 \x01(\t"C\n\x15\x42\x46TProducerBundleItem\x12*\n\tProducers\x18\x01 \x03(\x0b\x32\x17.quorum.pb.ProducerItem"\xc0\x01\n\x08UserItem\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x12\n\nUserPubkey\x18\x02 \x01(\t\x12\x15\n\rEncryptPubkey\x18\x03 \x01(\t\x12\x18\n\x10GroupOwnerPubkey\x18\x04 \x01(\t\x12\x16\n\x0eGroupOwnerSign\x18\x05 \x01(\t\x12\x11\n\tTimeStamp\x18\x06 \x01(\x03\x12%\n\x06\x41\x63tion\x18\x07 \x01(\x0e\x32\x15.quorum.pb.ActionType\x12\x0c\n\x04Memo\x18\x08 \x01(\t"\xaa\x02\n\x0c\x41nnounceItem\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x12\n\nSignPubkey\x18\x02 \x01(\t\x12\x15\n\rEncryptPubkey\x18\x03 \x01(\t\x12\x1a\n\x12\x41nnouncerSignature\x18\x04 \x01(\t\x12%\n\x04Type\x18\x05 \x01(\x0e\x32\x17.quorum.pb.AnnounceType\x12\x13\n\x0bOwnerPubkey\x18\x06 \x01(\t\x12\x16\n\x0eOwnerSignature\x18\x07 \x01(\t\x12&\n\x06Result\x18\x08 \x01(\x0e\x32\x16.quorum.pb.ApproveType\x12\x11\n\tTimeStamp\x18\t \x01(\x03\x12%\n\x06\x41\x63tion\x18\n \x01(\x0e\x32\x15.quorum.pb.ActionType\x12\x0c\n\x04Memo\x18\x0b \x01(\t"\xbc\x02\n\tGroupItem\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x11\n\tGroupName\x18\x02 \x01(\t\x12\x13\n\x0bOwnerPubKey\x18\x03 \x01(\t\x12\x16\n\x0eUserSignPubkey\x18\x04 \x01(\t\x12\x19\n\x11UserEncryptPubkey\x18\x05 \x01(\t\x12\x12\n\nLastUpdate\x18\x06 \x01(\x03\x12&\n\x0cGenesisBlock\x18\x07 \x01(\x0b\x32\x10.quorum.pb.Block\x12\x30\n\x0b\x45ncryptType\x18\x08 \x01(\x0e\x32\x1b.quorum.pb.GroupEncryptType\x12\x32\n\x0c\x43onsenseType\x18\t \x01(\x0e\x32\x1c.quorum.pb.GroupConsenseType\x12\x11\n\tCipherKey\x18\n \x01(\t\x12\x0e\n\x06\x41ppKey\x18\x0b \x01(\t"\xa8\x01\n\x0f\x43hainConfigItem\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12(\n\x04Type\x18\x02 \x01(\x0e\x32\x1a.quorum.pb.ChainConfigType\x12\x0c\n\x04\x44\x61ta\x18\x03 \x01(\x0c\x12\x13\n\x0bOwnerPubkey\x18\x04 \x01(\t\x12\x16\n\x0eOwnerSignature\x18\x05 \x01(\t\x12\x11\n\tTimeStamp\x18\x06 \x01(\x03\x12\x0c\n\x04Memo\x18\x07 \x01(\t"s\n\x18\x43hainSendTrxRuleListItem\x12%\n\x06\x41\x63tion\x18\x01 \x01(\x0e\x32\x15.quorum.pb.ActionType\x12\x0e\n\x06Pubkey\x18\x03 \x01(\t\x12 \n\x04Type\x18\x04 \x03(\x0e\x32\x12.quorum.pb.TrxType"\\\n\x12SetTrxAuthModeItem\x12 \n\x04Type\x18\x01 \x01(\x0e\x32\x12.quorum.pb.TrxType\x12$\n\x04Mode\x18\x02 \x01(\x0e\x32\x16.quorum.pb.TrxAuthMode"\xd5\x01\n\rAppConfigItem\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12%\n\x06\x41\x63tion\x18\x02 \x01(\x0e\x32\x15.quorum.pb.ActionType\x12\x0c\n\x04Name\x18\x03 \x01(\t\x12&\n\x04Type\x18\x04 \x01(\x0e\x32\x18.quorum.pb.AppConfigType\x12\r\n\x05Value\x18\x05 \x01(\t\x12\x13\n\x0bOwnerPubkey\x18\x06 \x01(\t\x12\x11\n\tOwnerSign\x18\x07 \x01(\t\x12\x0c\n\x04Memo\x18\x08 \x01(\t\x12\x11\n\tTimeStamp\x18\t \x01(\x03"\xd1\x01\n\tGroupSeed\x12&\n\x0cGenesisBlock\x18\x01 \x01(\x0b\x32\x10.quorum.pb.Block\x12\x0f\n\x07GroupId\x18\x02 \x01(\t\x12\x11\n\tGroupName\x18\x03 \x01(\t\x12\x13\n\x0bOwnerPubkey\x18\x04 \x01(\t\x12\x15\n\rConsensusType\x18\x05 \x01(\t\x12\x16\n\x0e\x45ncryptionType\x18\x06 \x01(\t\x12\x11\n\tCipherKey\x18\x07 \x01(\t\x12\x0e\n\x06\x41ppKey\x18\x08 \x01(\t\x12\x11\n\tSignature\x18\t \x01(\t"\x83\x01\n\x10NodeSDKGroupItem\x12#\n\x05Group\x18\x01 \x01(\x0b\x32\x14.quorum.pb.GroupItem\x12\x14\n\x0c\x45ncryptAlias\x18\x02 \x01(\t\x12\x11\n\tSignAlias\x18\x03 \x01(\t\x12\x0e\n\x06\x41piUrl\x18\x04 \x03(\t\x12\x11\n\tGroupSeed\x18\x05 \x01(\t"+\n\x0bHBTrxBundle\x12\x1c\n\x04Trxs\x18\x01 \x03(\x0b\x32\x0e.quorum.pb.Trx"j\n\x07HBMsgv1\x12\r\n\x05MsgId\x18\x01 \x01(\t\x12\r\n\x05\x45poch\x18\x02 \x01(\x04\x12\x30\n\x0bPayloadType\x18\x03 \x01(\x0e\x32\x1b.quorum.pb.HBMsgPayloadType\x12\x0f\n\x07Payload\x18\x04 \x01(\x0c">\n\x06RBCMsg\x12#\n\x04Type\x18\x01 \x01(\x0e\x32\x15.quorum.pb.RBCMsgType\x12\x0f\n\x07Payload\x18\x02 \x01(\x0c"\xad\x01\n\x0bInitPropose\x12\x10\n\x08RootHash\x18\x01 \x01(\x0c\x12\r\n\x05Proof\x18\x02 \x03(\x0c\x12\r\n\x05Index\x18\x03 \x01(\x03\x12\x0e\n\x06Leaves\x18\x04 \x01(\x03\x12\x18\n\x10OriginalDataSize\x18\x05 \x01(\x03\x12\x16\n\x0eRecvNodePubkey\x18\x06 \x01(\t\x12\x16\n\x0eProposerPubkey\x18\x07 \x01(\t\x12\x14\n\x0cProposerSign\x18\x08 \x01(\x0c"\xb6\x01\n\x04\x45\x63ho\x12\x10\n\x08RootHash\x18\x01 \x01(\x0c\x12\r\n\x05Proof\x18\x02 \x03(\x0c\x12\r\n\x05Index\x18\x03 \x01(\x03\x12\x0e\n\x06Leaves\x18\x04 \x01(\x03\x12\x18\n\x10OriginalDataSize\x18\x05 \x01(\x03\x12\x1e\n\x16OriginalProposerPubkey\x18\x06 \x01(\t\x12\x1a\n\x12\x45\x63hoProviderPubkey\x18\x07 \x01(\t\x12\x18\n\x10\x45\x63hoProviderSign\x18\x08 \x01(\x0c"q\n\x05Ready\x12\x10\n\x08RootHash\x18\x01 \x01(\x0c\x12\x1e\n\x16OriginalProposerPubkey\x18\x02 \x01(\t\x12\x1b\n\x13ReadyProviderPubkey\x18\x03 \x01(\t\x12\x19\n\x11ReadyProviderSign\x18\x04 \x01(\x0c">\n\x06\x42\x42\x41Msg\x12#\n\x04Type\x18\x01 \x01(\x0e\x32\x15.quorum.pb.BBAMsgType\x12\x0f\n\x07Payload\x18\x02 \x01(\x0c"N\n\x04\x42val\x12\x12\n\nProposerId\x18\x01 \x01(\t\x12\x14\n\x0cSenderPubkey\x18\x02 \x01(\t\x12\r\n\x05\x45poch\x18\x03 \x01(\x03\x12\r\n\x05Value\x18\x04 \x01(\x08"M\n\x03\x41ux\x12\x12\n\nProposerId\x18\x01 \x01(\t\x12\x14\n\x0cSenderPubkey\x18\x02 \x01(\t\x12\r\n\x05\x45poch\x18\x03 \x01(\x04\x12\r\n\x05Value\x18\x04 \x01(\x08"E\n\x08PSyncMsg\x12(\n\x07MsgType\x18\x01 \x01(\x0e\x32\x17.quorum.pb.PSyncMsgType\x12\x0f\n\x07Payload\x18\x02 \x01(\x0c"i\n\x08PSyncReq\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x11\n\tSessionId\x18\x02 \x01(\t\x12\x14\n\x0cSenderPubkey\x18\x03 \x01(\t\x12\x0f\n\x07MyEpoch\x18\x04 \x01(\x04\x12\x12\n\nSenderSign\x18\x05 \x01(\x0c"\xcd\x01\n\tPSyncResp\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x11\n\tSessionId\x18\x02 \x01(\t\x12\x14\n\x0cSenderPubkey\x18\x03 \x01(\t\x12\x12\n\nMyCurEpoch\x18\x04 \x01(\x04\x12\x37\n\x11MyCurProducerList\x18\x05 \x01(\x0b\x32\x1c.quorum.pb.PSyncProducerItem\x12%\n\rProducerProof\x18\x06 \x01(\x0b\x32\x0e.quorum.pb.Trx\x12\x12\n\nSenderSign\x18\x07 \x01(\x0c"&\n\x11PSyncProducerItem\x12\x11\n\tProducers\x18\x01 \x03(\t"\x92\x03\n\x0bGroupItemV0\x12\x0f\n\x07GroupId\x18\x01 \x01(\t\x12\x11\n\tGroupName\x18\x02 \x01(\t\x12\x13\n\x0bOwnerPubKey\x18\x03 \x01(\t\x12\x16\n\x0eUserSignPubkey\x18\x04 \x01(\t\x12\x19\n\x11UserEncryptPubkey\x18\x05 \x01(\t\x12#\n\x08UserRole\x18\x06 \x01(\x0e\x32\x11.quorum.pb.RoleV0\x12\x12\n\nLastUpdate\x18\x07 \x01(\x03\x12\x15\n\rHighestHeight\x18\x08 \x01(\x03\x12\x16\n\x0eHighestBlockId\x18\t \x01(\t\x12&\n\x0cGenesisBlock\x18\n \x01(\x0b\x32\x10.quorum.pb.Block\x12\x30\n\x0b\x45ncryptType\x18\x0b \x01(\x0e\x32\x1b.quorum.pb.GroupEncryptType\x12\x32\n\x0c\x43onsenseType\x18\x0c \x01(\x0e\x32\x1c.quorum.pb.GroupConsenseType\x12\x11\n\tCipherKey\x18\r \x01(\t\x12\x0e\n\x06\x41ppKey\x18\x0e \x01(\t*5\n\x0bPackageType\x12\x07\n\x03TRX\x10\x00\x12\t\n\x05\x42LOCK\x10\x01\x12\x07\n\x03HBB\x10\x02\x12\t\n\x05PSYNC\x10\x03*,\n\x0c\x41nnounceType\x12\x0b\n\x07\x41S_USER\x10\x00\x12\x0f\n\x0b\x41S_PRODUCER\x10\x01*8\n\x0b\x41pproveType\x12\r\n\tANNOUNCED\x10\x00\x12\x0c\n\x08\x41PPROVED\x10\x01\x12\x0c\n\x08REJECTED\x10\x02*!\n\nActionType\x12\x07\n\x03\x41\x44\x44\x10\x00\x12\n\n\x06REMOVE\x10\x01*&\n\x0eTrxStroageType\x12\t\n\x05\x43HAIN\x10\x00\x12\t\n\x05\x43\x41\x43HE\x10\x01*~\n\x07TrxType\x12\x08\n\x04POST\x10\x00\x12\x0c\n\x08\x41NNOUNCE\x10\x01\x12\x0c\n\x08PRODUCER\x10\x02\x12\x08\n\x04USER\x10\x03\x12\r\n\tREQ_BLOCK\x10\x04\x12\x12\n\x0eREQ_BLOCK_RESP\x10\x05\x12\x10\n\x0c\x43HAIN_CONFIG\x10\x06\x12\x0e\n\nAPP_CONFIG\x10\x07*P\n\x0cReqBlkResult\x12\x11\n\rBLOCK_IN_RESP\x10\x00\x12\x18\n\x14\x42LOCK_IN_RESP_ON_TOP\x10\x01\x12\x13\n\x0f\x42LOCK_NOT_FOUND\x10\x02*+\n\x10GroupEncryptType\x12\n\n\x06PUBLIC\x10\x00\x12\x0b\n\x07PRIVATE\x10\x01*%\n\x11GroupConsenseType\x12\x07\n\x03POA\x10\x00\x12\x07\n\x03POS\x10\x01*,\n\x06RoleV0\x12\x12\n\x0eGROUP_PRODUCER\x10\x00\x12\x0e\n\nGROUP_USER\x10\x01*L\n\x0f\x43hainConfigType\x12\x15\n\x11SET_TRX_AUTH_MODE\x10\x00\x12\x10\n\x0cUPD_DNY_LIST\x10\x01\x12\x10\n\x0cUPD_ALW_LIST\x10\x02*7\n\x0bTrxAuthMode\x12\x13\n\x0f\x46OLLOW_ALW_LIST\x10\x00\x12\x13\n\x0f\x46OLLOW_DNY_LIST\x10\x01*-\n\x0c\x41uthListType\x12\x0e\n\nALLOW_LIST\x10\x00\x12\r\n\tDENY_LIST\x10\x01*.\n\rAppConfigType\x12\x07\n\x03INT\x10\x00\x12\x08\n\x04\x42OOL\x10\x01\x12\n\n\x06STRING\x10\x02*$\n\x10HBMsgPayloadType\x12\x07\n\x03RBC\x10\x00\x12\x07\n\x03\x42\x42\x41\x10\x01*3\n\nRBCMsgType\x12\x10\n\x0cINIT_PROPOSE\x10\x00\x12\x08\n\x04\x45\x43HO\x10\x01\x12\t\n\x05READY\x10\x02*\x1f\n\nBBAMsgType\x12\x08\n\x04\x42VAL\x10\x00\x12\x07\n\x03\x41UX\x10\x01*-\n\x0cPSyncMsgType\x12\r\n\tPSYNC_REQ\x10\x00\x12\x0e\n\nPSYNC_RESP\x10\x01\x42$Z"github.com/rumsystem/quorum/pkg/pbb\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "quorum_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'Z"github.com/rumsystem/quorum/pkg/pb'
    _PACKAGETYPE._serialized_start = 4795
    _PACKAGETYPE._serialized_end = 4848
    _ANNOUNCETYPE._serialized_start = 4850
    _ANNOUNCETYPE._serialized_end = 4894
    _APPROVETYPE._serialized_start = 4896
    _APPROVETYPE._serialized_end = 4952
    _ACTIONTYPE._serialized_start = 4954
    _ACTIONTYPE._serialized_end = 4987
    _TRXSTROAGETYPE._serialized_start = 4989
    _TRXSTROAGETYPE._serialized_end = 5027
    _TRXTYPE._serialized_start = 5029
    _TRXTYPE._serialized_end = 5155
    _REQBLKRESULT._serialized_start = 5157
    _REQBLKRESULT._serialized_end = 5237
    _GROUPENCRYPTTYPE._serialized_start = 5239
    _GROUPENCRYPTTYPE._serialized_end = 5282
    _GROUPCONSENSETYPE._serialized_start = 5284
    _GROUPCONSENSETYPE._serialized_end = 5321
    _ROLEV0._serialized_start = 5323
    _ROLEV0._serialized_end = 5367
    _CHAINCONFIGTYPE._serialized_start = 5369
    _CHAINCONFIGTYPE._serialized_end = 5445
    _TRXAUTHMODE._serialized_start = 5447
    _TRXAUTHMODE._serialized_end = 5502
    _AUTHLISTTYPE._serialized_start = 5504
    _AUTHLISTTYPE._serialized_end = 5549
    _APPCONFIGTYPE._serialized_start = 5551
    _APPCONFIGTYPE._serialized_end = 5597
    _HBMSGPAYLOADTYPE._serialized_start = 5599
    _HBMSGPAYLOADTYPE._serialized_end = 5635
    _RBCMSGTYPE._serialized_start = 5637
    _RBCMSGTYPE._serialized_end = 5688
    _BBAMSGTYPE._serialized_start = 5690
    _BBAMSGTYPE._serialized_end = 5721
    _PSYNCMSGTYPE._serialized_start = 5723
    _PSYNCMSGTYPE._serialized_end = 5768
    _PACKAGE._serialized_start = 27
    _PACKAGE._serialized_end = 88
    _TRX._serialized_start = 91
    _TRX._serialized_end = 355
    _BLOCK._serialized_start = 358
    _BLOCK._serialized_end = 560
    _REQBLOCK._serialized_start = 562
    _REQBLOCK._serialized_end = 650
    _BLOCKSBUNDLE._serialized_start = 652
    _BLOCKSBUNDLE._serialized_end = 700
    _REQBLOCKRESP._serialized_start = 703
    _REQBLOCKRESP._serialized_end = 929
    _POSTITEM._serialized_start = 931
    _POSTITEM._serialized_end = 1014
    _PRODUCERITEM._serialized_start = 1017
    _PRODUCERITEM._serialized_end = 1218
    _BFTPRODUCERBUNDLEITEM._serialized_start = 1220
    _BFTPRODUCERBUNDLEITEM._serialized_end = 1287
    _USERITEM._serialized_start = 1290
    _USERITEM._serialized_end = 1482
    _ANNOUNCEITEM._serialized_start = 1485
    _ANNOUNCEITEM._serialized_end = 1783
    _GROUPITEM._serialized_start = 1786
    _GROUPITEM._serialized_end = 2102
    _CHAINCONFIGITEM._serialized_start = 2105
    _CHAINCONFIGITEM._serialized_end = 2273
    _CHAINSENDTRXRULELISTITEM._serialized_start = 2275
    _CHAINSENDTRXRULELISTITEM._serialized_end = 2390
    _SETTRXAUTHMODEITEM._serialized_start = 2392
    _SETTRXAUTHMODEITEM._serialized_end = 2484
    _APPCONFIGITEM._serialized_start = 2487
    _APPCONFIGITEM._serialized_end = 2700
    _GROUPSEED._serialized_start = 2703
    _GROUPSEED._serialized_end = 2912
    _NODESDKGROUPITEM._serialized_start = 2915
    _NODESDKGROUPITEM._serialized_end = 3046
    _HBTRXBUNDLE._serialized_start = 3048
    _HBTRXBUNDLE._serialized_end = 3091
    _HBMSGV1._serialized_start = 3093
    _HBMSGV1._serialized_end = 3199
    _RBCMSG._serialized_start = 3201
    _RBCMSG._serialized_end = 3263
    _INITPROPOSE._serialized_start = 3266
    _INITPROPOSE._serialized_end = 3439
    _ECHO._serialized_start = 3442
    _ECHO._serialized_end = 3624
    _READY._serialized_start = 3626
    _READY._serialized_end = 3739
    _BBAMSG._serialized_start = 3741
    _BBAMSG._serialized_end = 3803
    _BVAL._serialized_start = 3805
    _BVAL._serialized_end = 3883
    _AUX._serialized_start = 3885
    _AUX._serialized_end = 3962
    _PSYNCMSG._serialized_start = 3964
    _PSYNCMSG._serialized_end = 4033
    _PSYNCREQ._serialized_start = 4035
    _PSYNCREQ._serialized_end = 4140
    _PSYNCRESP._serialized_start = 4143
    _PSYNCRESP._serialized_end = 4348
    _PSYNCPRODUCERITEM._serialized_start = 4350
    _PSYNCPRODUCERITEM._serialized_end = 4388
    _GROUPITEMV0._serialized_start = 4391
    _GROUPITEMV0._serialized_end = 4793
# @@protoc_insertion_point(module_scope)



import base64
import json
import logging
import os
import uuid

from pyrage import decrypt, encrypt, passphrase, x25519

logger = logging.getLogger(__name__)


def create_age_keypair():
    """create age private key"""
    age_identity = x25519.Identity.generate()
    age_pvtkey = str(age_identity)
    age_pubkey = str(age_identity.to_public())
    return age_pvtkey, age_pubkey


def age_pubkey_from_str(pubkey: str) -> x25519.Recipient:
    return x25519.Recipient.from_str(pubkey)


def age_pvtkey_from_str(pvtkey: str) -> x25519.Identity:
    return x25519.Identity.from_str(pvtkey)


def age_pvtkey_to_pubkey(pvtkey: str) -> str:
    """age private key to public key"""
    identity = age_pvtkey_from_str(pvtkey)
    pubkey = str(identity.to_public())
    return pubkey


def age_pvtkey_from_file(path: str, password: str) -> x25519.Identity:
    if not os.path.exists(path):
        raise ValueError(f"can not find file path: {path}")

    with open(path, "rb") as fp:
        decrypted = passphrase.decrypt(fp.read(), password)
    return age_pvtkey_from_str(str(decrypted))


def age_encrypt(recipients: list, data: bytes) -> bytes:
    """recipients list of x25519.Recipient"""
    if not data:
        raise ValueError("invalid data")
    if not recipients:
        raise ValueError("invalid recipients")

    _recipients = []
    for item in recipients:
        if isinstance(item, str):
            _recipients.append(age_pubkey_from_str(item))
        elif isinstance(item, x25519.Recipient):
            _recipients.append(item)
        else:
            raise ValueError(f"invalid recipients {item}")

    return encrypt(data, _recipients)


def age_decrypt(identity: x25519.Identity, data: bytes):
    if isinstance(identity, str):
        identity = age_pvtkey_from_str(identity)
    elif not isinstance(identity, x25519.Identity):
        raise ValueError("invalid identity")

    try:
        return decrypt(data, [identity])
    except Exception as err:
        logger.warning("age_decrypt error: %s", err)
        return None


def trx_age_encrypt(data: dict, recipients: list, post_id: str = None):
    data = json.dumps(data).encode()
    encrypted = age_encrypt(recipients, data)
    return {
        "content": base64.b64encode(encrypted).decode(),
        "type": "age",
        "post_id": post_id or str(uuid.uuid4()),
        "recipients": recipients,
    }


def trx_age_decrypt(data: dict, identity: x25519.Identity):
    if isinstance(data, str):
        rlt = {"content": data}
        content = data
    if isinstance(data, dict):
        rlt = data.copy()
        content = data.get("content")
    if not content:
        raise ValueError("invalid content")
    recipients = data.get("recipients", [])
    if recipients and age_pvtkey_to_pubkey(identity) not in recipients:
        return data

    decrypted = age_decrypt(identity, base64.b64decode(content))
    rlt["content"] = json.loads(decrypted.decode())
    return rlt



import logging
import os

import requests

logger = logging.getLogger(__name__)


class HttpRequest:
    """Class for making http requests"""

    def __init__(
        self,
        api_base: str = None,
        jwt_token: str = None,
        chain_urls: list = None,
        timeout: int = 5,
        retries: int = 3,
    ):
        """
        Initializes the HttpRequest class
        chain_urls: list of dict of api_base and jwt_token
        """
        requests.adapters.DEFAULT_RETRIES = 5
        if api_base:
            chain_urls = [{"baseurl": api_base, "jwt": jwt_token}]
        self.chain_urls = chain_urls
        self.current_chain_url = 0
        self.session = requests.Session()
        self.timeout = timeout
        self.retries = retries
        self.headers = {"Content-Type": "application/json"}
        self.session.headers.update(self.headers)

    def _request(
        self,
        method: str,
        endpoint: str,
        payload: dict = None,
    ):
        chain_url = self.chain_urls[self.current_chain_url]
        api_base = chain_url.get("baseurl")
        jwt_token = chain_url.get("jwt")
        if jwt_token:
            self.headers.update({"Authorization": f"Bearer {jwt_token}"})
            self.session.headers.update(self.headers)
        url = api_base + endpoint

        _no_proxy = os.getenv("NO_PROXY", "")
        if api_base not in _no_proxy:
            os.environ["NO_PROXY"] = ",".join([_no_proxy, api_base])

        try:
            for i in range(self.retries):
                try:
                    response = self.session.request(
                        method=method,
                        url=url,
                        json=payload,
                        timeout=self.timeout,
                        headers=self.headers,
                    )
                    logger.debug("payload %s", payload)
                    response.raise_for_status()
                    return response.json()
                except requests.exceptions.RequestException as err:
                    logging.warning(
                        "HTTP request retry %s/%s: %s", i + 1, self.retries, err
                    )
        except requests.exceptions.RequestException as err:
            logging.warning(
                "HTTP request error (endpoint %s}): %s", self.current_chain_url, err
            )
            logger.debug("payload %s", payload)
            self.current_chain_url = (self.current_chain_url + 1) % len(self.chain_urls)
            return self._request(method, endpoint, payload)
        raise Exception("HTTP request error")

    def get(self, endpoint: str, payload: dict = None):
        return self._request("get", endpoint, payload)

    def post(self, endpoint: str, payload: dict = None):
        return self._request("post", endpoint, payload)



import logging
import os

from Crypto.Cipher import AES

logger = logging.getLogger(__name__)


def aes_encrypt(key: bytes, data: bytes) -> bytes:
    cipher = AES.new(key, AES.MODE_GCM, nonce=os.urandom(12))
    ciphertext, tag = cipher.encrypt_and_digest(data)
    return b"".join([cipher.nonce, ciphertext, tag])


def aes_decrypt(key: bytes, data: bytes) -> bytes:
    nonce, tag = data[:12], data[-16:]
    cipher = AES.new(key, AES.MODE_GCM, nonce=nonce)
    return cipher.decrypt_and_verify(data[12:-16], tag)



import logging

from quorum_mininode_py.api.base import BaseAPI
from quorum_mininode_py.crypto.account import check_pvtkey
from quorum_mininode_py.crypto.trx import get_announce_param, trx_decrypt, trx_encrypt

logger = logging.getLogger(__name__)


class LightNodeAPI(BaseAPI):
    """the light node api for quorum"""

    def post_content(
        self,
        data: dict,
        trx_id: str = None,
    ):
        """post content to group"""
        trx = trx_encrypt(
            self.group_id,
            self._group.aes_key,
            data,
            check_pvtkey(self._account.pvtkey),
            trx_id,
        )
        return super()._post_content(trx)

    def get_trx(self, trx_id: str):
        """get encrpyted trx"""
        return super()._get_trx(trx_id)

    def trx(self, trx_id: str):
        """get decrypted trx"""
        return trx_decrypt(self._group.aes_key, self.get_trx(trx_id))

    def get_content(
        self,
        start_trx: str = None,
        num: int = 20,
        reverse: bool = False,
        include_start_trx: bool = False,
        senders: list = None,
    ):
        """get content"""
        params = {
            "reverse": reverse,
            "num": num,
        }
        if start_trx:
            params["start_trx"] = start_trx
            params["include_start_trx"] = include_start_trx
        if senders:
            params["senders"] = senders

        encypted_trxs = super()._get_content(params)
        trxs = []
        for trx in encypted_trxs:
            try:
                trx = trx_decrypt(self._group.aes_key, trx)
            except Exception as err:
                logger.error(err)
            trxs.append(trx)
        return trxs

    def get_group_info(self):
        """get group info"""
        return super()._get_group()

    def get_encryptpubkeys(self):
        """get encrypt pubkeys"""
        return super()._get_encryptpubkeys()

    def get_auth_type(self, trx_type: str):
        """get auth type of trx_type"""
        trx_type = trx_type.upper()
        if trx_type not in ["POST", "ANNOUNCE", "REQ_BLOCK"]:
            raise ValueError("trx_type must be one of [POST ANNOUNCE REQ_BLOCK]")
        return super()._get_auth_type(trx_type)

    def get_auth_allowlist(self):
        """get allowlist"""
        return super()._get_alwlist()

    def get_auth_denylist(self):
        """get denylist"""
        return super()._get_denylist()

    def get_appconfig_keylist(self):
        """get appconfig keylist"""
        return super()._get_appconfig_keylist()

    def get_appconfig_key(self, key: str):
        """get appconfig key value by keyname"""
        return super()._get_appconfig_key(key)

    def get_producers(self):
        """get group producers"""
        return super()._get_producers()

    def get_announced_producer(self):
        """get announced producer"""
        return super()._get_announced_producer()

    def get_announced_user(self):
        """get announced user"""
        return super()._get_announced_user()

    def announce(self, action: str, _type: str, memo: str = None):
        """post annonce to group
        action: str, one of [add, remove]
        _type: str, one of [user, producer]
        """
        # only announce self as user will be success
        payload = get_announce_param(
            self._account.age_pubkey,
            check_pvtkey(self._account.pvtkey),
            self.group_id,
            action,
            _type,
            memo or f"annonce as {_type}",
        )
        return super()._post_announce(payload)

    def announce_as_user(self):
        """announce as user"""
        return self.announce("add", "user")

    def announce_as_producer(self):
        """announce as producer"""
        # lightnote can not announce as producer, return http 400
        return self.announce("add", "producer")

    def get_encrypt_pubkeys(self):
        """get age pubkeys of private group"""
        return super()._get_encryptpubkeys().get("keys")



import base64
import hashlib
import json
import logging
import time
import uuid
from typing import Any, Dict, Union

import eth_keys
from google.protobuf.json_format import MessageToDict

from quorum_mininode_py.crypto.account import pvtkey_to_pubkey
from quorum_mininode_py.crypto.aes import aes_decrypt, aes_encrypt
from quorum_mininode_py.proto import pbQuorum

logger = logging.getLogger(__name__)

# pylint: disable=no-member


def trx_encrypt(
    group_id: str,
    aes_key: bytes,
    data: Dict[str, Any] = None,
    private_key: bytes = None,
    trx_id: str = None,
) -> Dict[str, str]:
    """trx encrypt"""
    # pylint: disable=too-many-locals
    data = json.dumps(data).encode()
    encrypted = aes_encrypt(aes_key, data)

    trx = {
        "TrxId": trx_id or str(uuid.uuid4()),
        "GroupId": group_id,
        "Data": encrypted,
        "TimeStamp": int(time.time() * 1e9),
        "Version": "2.0.0",
        "SenderPubkey": pvtkey_to_pubkey(private_key),
    }

    trx_without_sign_pb = pbQuorum.Trx(**trx)  # pylint: disable=no-member
    trx_without_sign_pb_bytes = trx_without_sign_pb.SerializeToString()
    trx_hash = hashlib.sha256(trx_without_sign_pb_bytes).digest()
    account = eth_keys.keys.PrivateKey(private_key)
    trx["SenderSign"] = account.sign_msg_hash(trx_hash).to_bytes()

    for k, v in trx.items():
        if isinstance(v, bytes):
            trx[k] = base64.b64encode(v).decode()

    trx = {
        "trx_id": trx["TrxId"],
        "data": trx["Data"],
        "timestamp": str(trx["TimeStamp"]),
        "version": trx["Version"],
        "sender_pubkey": trx["SenderPubkey"],
        "sender_sign": trx["SenderSign"],
    }
    return trx


def trx_decrypt(aes_key: Union[bytes, None], encrypted_trx: dict):
    if "trx" in encrypted_trx:
        encrypted_trx = encrypted_trx["trx"]
    data = encrypted_trx.get("Data")
    if data is None:
        raise ValueError("Data is None")
    trx_enc_bytes = base64.b64decode(data)
    trx_bytes = aes_decrypt(aes_key, trx_enc_bytes)
    trx = {**encrypted_trx, "Data": json.loads(trx_bytes)}
    return trx


def get_announce_param(
    age_pubkey: str,
    private_key: bytes,
    group_id: str,
    action: str,
    _type: str,
    memo: Union[str, None] = None,
) -> Dict[str, Dict[str, Any]]:
    item = pbQuorum.AnnounceItem()
    item.GroupId = group_id
    if _type == "user":
        item.Type = pbQuorum.AS_USER
    elif _type == "producer":
        item.Type = pbQuorum.AS_PRODUCER
    else:
        raise ValueError("unsupport type")

    if action == "add":
        item.Action = pbQuorum.ADD
    elif action == "remove":
        item.Action = pbQuorum.REMOVE
    else:
        raise ValueError("unsupport action")

    item.SignPubkey = pvtkey_to_pubkey(private_key)
    if item.Type == pbQuorum.AS_USER:
        item.EncryptPubkey = age_pubkey

    item.OwnerPubkey = ""
    item.OwnerSignature = ""
    item.Result = pbQuorum.ANNOUNCED

    data = (
        item.GroupId
        + item.SignPubkey
        + item.EncryptPubkey
        + pbQuorum.AnnounceType.Name(item.Type)
    ).encode()
    _hash = hashlib.sha256(data).digest()
    signature = eth_keys.keys.PrivateKey(private_key).sign_msg_hash(_hash).to_bytes()

    item.AnnouncerSignature = signature.hex()
    item.TimeStamp = int(time.time() * 1e9)
    item.Memo = memo or "lightnode announce"

    payload = {
        "data": MessageToDict(item),
    }
    return payload



import base64
import hashlib
import uuid
from urllib.parse import parse_qs, unquote, urlencode, urlparse

from google.protobuf import json_format

from quorum_mininode_py.proto import pbQuorum


def urlsafe_b64decode(b64str: str) -> bytes:
    num = (4 - len(b64str) % 4) % 4
    b64byte = b64str.encode() + b"=" * num
    b64byte = base64.urlsafe_b64decode(b64byte)
    return b64byte


def _get_value_from_query(query: dict, key: str) -> str:
    val = query.get(key)
    if not val:
        raise KeyError(f"can not find key: {key} or value is empty")
    if not isinstance(val, list):
        raise ValueError("value is not a list")
    return val[0]


def extract_uuid_from_query(query: dict, key: str) -> str:
    val = _get_value_from_query(query, key)
    b = urlsafe_b64decode(val)
    return str(uuid.UUID(bytes=b))


def extra_timestamp_from_query(query: dict, key: str) -> int:
    val = _get_value_from_query(query, key)
    t = urlsafe_b64decode(val)
    timestamp = int.from_bytes(t, byteorder="big")
    return timestamp


def hash_block(block: pbQuorum.Block) -> bytes:  # pylint: disable=no-member
    new_block = pbQuorum.Block()  # pylint: disable=no-member
    new_block.CopyFrom(block)
    new_block.BlockHash = b""
    new_block.ProducerSign = b""
    new_block_bytes = new_block.SerializeToString()
    return hashlib.sha256(new_block_bytes).digest()


def parse_chain_url(url: str):
    u = urlparse(url)
    baseurl = f"{u.scheme}://{u.hostname}:{u.port}"
    query = parse_qs(u.query)
    jwt = _get_value_from_query(query, "jwt")
    return dict(baseurl=baseurl, jwt=jwt)


def decode_seed_url(seed_url: str):
    """
    seed_url (str):
    the seed url of rum group which shared by rum fullnode,
    with host:post?jwt=xxx to connect
    """

    if not isinstance(seed_url, str):
        raise TypeError("seed_url must be string type.")

    if not seed_url.startswith("rum://seed?v=1"):
        raise ValueError("seed_url must start with rum://seed?")

    parsed = urlparse(seed_url)
    query = parse_qs(parsed.query)

    group_id = extract_uuid_from_query(query, "g")
    genesis_block = pbQuorum.Block(  # pylint: disable=no-member
        Epoch=0,
        GroupId=group_id,
        PrevHash=None,
        Trxs=None,
    )
    genesis_block.BlockHash = hash_block(genesis_block)
    genesis_block.ProducerPubkey = _get_value_from_query(query, "k")
    genesis_block.TimeStamp = extra_timestamp_from_query(query, "t")
    genesis_block.BlockHash = hash_block(genesis_block)
    genesis_block.ProducerSign = urlsafe_b64decode(_get_value_from_query(query, "s"))

    consensus_type = "pos" if _get_value_from_query(query, "n") == "1" else "poa"
    encryption_type = (
        "public" if _get_value_from_query(query, "e") == "0" else "private"
    )
    seed = dict(
        genesis_block=genesis_block,
        group_id=group_id,
        group_name=_get_value_from_query(query, "a"),
        consensus_type=consensus_type,
        encryption_type=encryption_type,
        cipher_key=urlsafe_b64decode(_get_value_from_query(query, "c")).hex(),
        owner_pubkey=genesis_block.ProducerPubkey,
        signature=genesis_block.ProducerSign.hex(),
        app_key=unquote(_get_value_from_query(query, "y")),
    )

    chain_urls = []
    urls = list(set(_get_value_from_query(query, "u").split("|")))
    for url in urls:
        item = parse_chain_url(url)
        chain_urls.append(item)

    seed["genesis_block"] = json_format.MessageToDict(seed["genesis_block"])

    # get clean seed_url without chain_url
    query["u"] = [""]
    new_query_str = urlencode(query, doseq=True)
    updated_url = f"{parsed.scheme}://{parsed.netloc}{parsed.path}?{new_query_str}"

    return dict(seed=seed, chain_urls=chain_urls, seed_url=updated_url)


def update_seed_url(seed_url: str, *chain_urls: list):
    """
    seed_url (str):
    the seed url of rum group which shared by rum fullnode,
    with host:post?jwt=xxx to connect
    chain_urls (list):
    the chain urls of rum group which shared by rum fullnode,
    with host:post?jwt=xxx to connect
    """
    decoded = decode_seed_url(seed_url)
    for chain_url in chain_urls:
        if chain_url not in decoded["chain_urls"]:
            decoded["chain_urls"].append(chain_url)

    chain_url_str = ""
    for url in decoded["chain_urls"]:
        chain_url_str += f"{url['baseurl']}?jwt={url['jwt']}|"
    if chain_url_str.endswith("|"):
        chain_url_str = chain_url_str[:-1]
    updated_url = decoded["seed_url"] + "&u=" + chain_url_str
    return updated_url
